@@ -51,6 +51,10 @@ static void        mousepad_application_open                      (GApplication
 static void        mousepad_application_shutdown                  (GApplication             *gapplication);
 
 /* MousepadApplication own functions */
+static gboolean    mousepad_application_parse_encoding            (const gchar              *option_name,
+                                                                   const gchar              *value,
+                                                                   gpointer                  data,
+                                                                   GError                  **error);
 static GtkWidget  *mousepad_application_create_window             (MousepadApplication      *application);
 static void        mousepad_application_new_window_with_document  (MousepadWindow           *existing,
                                                                    MousepadDocument         *document,
@@ -95,8 +99,9 @@ struct _MousepadApplication
   GtkWidget                   *prefs_dialog;
   GtkSourceSpaceLocationFlags  space_location_flags;
 
-  /* opening mode provided on the command line */
-  gint                         opening_mode;
+  /* command line options */
+  gint             opening_mode;
+  MousepadEncoding encoding;
 };
 
 /* MousepadApplication properties */
@@ -111,6 +116,17 @@ enum
 /* command line options */
 static const GOptionEntry option_entries[] =
 {
+  {
+    "encoding", 'e', G_OPTION_FLAG_OPTIONAL_ARG,
+    G_OPTION_ARG_CALLBACK, mousepad_application_parse_encoding,
+    N_("Encoding to be used to open files (leave empty to open files in the encoding dialog)"),
+    N_("ENCODING")
+  },
+  {
+    "list-encodings", '\0', G_OPTION_FLAG_NONE,
+    G_OPTION_ARG_NONE, NULL,
+    N_("Display a list of possible encodings to open files"), NULL
+  },
   {
     "disable-server", '\0', G_OPTION_FLAG_NONE,
     G_OPTION_ARG_NONE, NULL,
@@ -295,6 +311,15 @@ mousepad_application_init (MousepadApplication *application)
 {
   gchar *option_desc;
 
+  /* initialize mousepad settings */
+  mousepad_settings_init ();
+
+  /* initialize application attributes */
+  application->prefs_dialog = NULL;
+  application->space_location_flags = GTK_SOURCE_SPACE_LOCATION_ALL;
+  application->opening_mode = TAB;
+  application->encoding = mousepad_encoding_get_default ();
+
   /* default application name */
   g_set_application_name (_("Mousepad"));
 
@@ -317,24 +342,64 @@ mousepad_application_init (MousepadApplication *application)
 
 
 
+static gboolean
+mousepad_application_parse_encoding (const gchar  *option_name,
+                                     const gchar  *value,
+                                     gpointer      data,
+                                     GError      **error)
+{
+  MousepadApplication *application;
+
+  application = MOUSEPAD_APPLICATION (g_application_get_default ());
+
+  /* we will redirect the user to the encoding dialog */
+  if (value == NULL)
+    application->encoding = MOUSEPAD_ENCODING_NONE;
+  else
+    {
+      /* try to find the encoding for the charset provided on the command line */
+      application->encoding = mousepad_encoding_find (value);
+
+      /* fallback to default if charset was invalid */
+      if (application->encoding == MOUSEPAD_ENCODING_NONE)
+        {
+          g_printerr ("Invalid encoding '%s': ignored\n", value);
+          application->encoding = mousepad_encoding_get_default ();
+        }
+    }
+
+  return TRUE;
+}
+
+
+
 static gint
 mousepad_application_handle_local_options (GApplication *gapplication,
                                            GVariantDict *options)
 {
+  MousepadEncoding   encoding;
   GApplicationFlags  flags;
   GError            *error = NULL;
 
   if (g_variant_dict_contains (options, "version"))
     {
       g_print ("%s %s\n\n", PACKAGE_NAME, PACKAGE_VERSION);
-      g_print ("%s\n", "Copyright (c) 2007");
-      g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
+      g_print ("%s\n", "Copyright (C) 2005-2021");
+      g_print ("\t%s\n\n", _("The Mousepad developers. All rights reserved."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
       g_print ("\n");
 
       return EXIT_SUCCESS;
     }
 
+  if (g_variant_dict_contains (options, "list-encodings"))
+    {
+      for (encoding = 1; encoding < MOUSEPAD_N_ENCODINGS; encoding++)
+        g_print ("%s\n", mousepad_encoding_get_charset (encoding));
+
+      return EXIT_SUCCESS;
+    }
+
   if (g_variant_dict_contains (options, "quit"))
     {
       /* try to register the application */
@@ -409,8 +474,7 @@ mousepad_application_complete_accel_map (GtkApplication *application)
   gchar        *accel_path, *filename;
   guint         n;
   gchar       **action_names;
-  const gchar  *accels[] = { NULL };
-  const gchar  *excluded_actions[] = { "win.edit.delete", "win.edit.select-all", "win.insensitive" };
+  const gchar  *excluded_actions[] = { "win.insensitive" };
 
   /* disconnect this handler */
   mousepad_disconnect_by_func (application, mousepad_application_complete_accel_map, NULL);
@@ -432,19 +496,10 @@ mousepad_application_complete_accel_map (GtkApplication *application)
 
   g_strfreev (action_names);
 
-  /*
-   * Some accels are already active at the widget level for some widgets, so we don't need and
-   * should not activate them at the action level, to not introduce conflicts.
-   * But we want them to appear in the menubar, as an indication for the user.
-   * So this trick does the job: adding them to the GtkBuilder UI file, and deactivating them
-   * when the menubar is set.
-  */
+  /* prevent actions that should not have a (configurable) keybinding from being saved
+   * in the accels file at shutdown */
   for (n = 0; n < G_N_ELEMENTS (excluded_actions); n++)
     {
-      /* deactivate accel in the application */
-      gtk_application_set_accels_for_action (GTK_APPLICATION (application), excluded_actions[n], accels);
-
-      /* prevent accel from being saved in the accels file at shutdown */
       accel_path = g_strconcat ("<Actions>/", excluded_actions[n], NULL);
       gtk_accel_map_add_filter (accel_path);
       g_free (accel_path);
@@ -483,12 +538,13 @@ mousepad_application_set_accels (MousepadApplication *application)
     { "win.file.open", "<Control>O" }, { "win.file.save", "<Control>S" },
     { "win.file.save-as", "<Control><Shift>S" }, { "win.file.print", "<Control>P" },
     { "win.file.detach-tab", "<Control>D" }, { "win.file.close-tab", "<Control>W" },
-    { "win.file.close-window", "<Control>Q" },
+    { "win.file.close-window", "<Control><Shift>W" }, { "app.quit", "<Control>Q" },
 
     /* "Edit" menu */
     { "win.edit.undo", "<Control>Z" }, { "win.edit.redo", "<Control>Y" },
     { "win.edit.cut", "<Control>X" }, { "win.edit.copy", "<Control>C" },
-    { "win.edit.paste", "<Control>V" }, { "win.edit.copy", "<Control>C" },
+    { "win.edit.paste", "<Control>V" }, { "win.edit.delete-selection", "Delete" },
+    { "win.edit.delete-line", "<Control><Shift>Delete" }, { "win.edit.select-all", "<Control>A" },
     { "win.edit.convert.to-opposite-case", "<Alt><Control>U" },
     { "win.edit.convert.transpose", "<Control>T" },
     { "win.edit.move-selection.line-up", "<Alt>Page_Up" },
@@ -497,14 +553,16 @@ mousepad_application_set_accels (MousepadApplication *application)
 
     /* "Search" menu */
     { "win.search.find", "<Control>F" }, { "win.search.find-next", "<Control>G" },
-    { "win.search.find-previous", "<Control><Shift>G" }, { "win.search.find-and-replace", "<Control>R" },
+    { "win.search.find-previous", "<Control><Shift>G" },
+    { "win.search.find-and-replace", "<Control>R" },
     { "win.search.go-to", "<Control>L" },
 
     /* "View" menu */
     { "win.preferences.window.menubar-visible", "<Control>M" }, { "win.view.fullscreen", "F11" },
 
     /* "Document" menu */
-    { "win.document.previous-tab", "<Control>Page_Up" }, { "win.document.next-tab", "<Control>Page_Down" },
+    { "win.document.previous-tab", "<Control>Page_Up" },
+    { "win.document.next-tab", "<Control>Page_Down" },
     { "win.document.go-to-tab(0)", "<Alt>1" }, { "win.document.go-to-tab(1)", "<Alt>2" },
     { "win.document.go-to-tab(2)", "<Alt>3" }, { "win.document.go-to-tab(3)", "<Alt>4" },
     { "win.document.go-to-tab(4)", "<Alt>5" }, { "win.document.go-to-tab(5)", "<Alt>6" },
@@ -565,14 +623,6 @@ mousepad_application_startup (GApplication *gapplication)
   /* chain up to parent */
   G_APPLICATION_CLASS (mousepad_application_parent_class)->startup (gapplication);
 
-  /* initialize application attributes */
-  application->prefs_dialog = NULL;
-  application->space_location_flags = GTK_SOURCE_SPACE_LOCATION_ALL;
-  application->opening_mode = TAB;
-
-  /* initialize mousepad settings */
-  mousepad_settings_init ();
-
   /* add application actions */
   g_action_map_add_action_entries (G_ACTION_MAP (application), stateless_actions,
                                    N_STATELESS, application);
@@ -720,19 +770,10 @@ static GtkWidget *
 mousepad_application_get_window_for_files (MousepadApplication *application)
 {
   GtkWidget *window = NULL;
-  GList     *list, *li;
 
   /* when opening mode is "tab", retrieve the last active MousepadWindow, if any */
   if (application->opening_mode == TAB)
-    {
-      list = gtk_application_get_windows (GTK_APPLICATION (application));
-      for (li = list; li != NULL; li = li->next)
-        if (MOUSEPAD_IS_WINDOW (li->data))
-          {
-            window = li->data;
-            break;
-          }
-    }
+    window = GTK_WIDGET (gtk_application_get_active_window (GTK_APPLICATION (application)));
 
   /* create a new window (signals added and already hooked up) if needed */
   if (window == NULL)
@@ -768,40 +809,24 @@ mousepad_application_open (GApplication  *gapplication,
 {
   MousepadApplication *application = MOUSEPAD_APPLICATION (gapplication);
   GtkWidget           *window;
-  GPtrArray           *uris;
-  gpointer            *data;
-  gchar               *filename, *uri;
-  gchar               *temp_uri[] = { NULL, NULL };
+  GFile               *valid_files[n_files];
+  gchar               *uri;
   gint                 n, valid = 0, opened = 0;
 
-  /* prepare the uris array */
-  uris = g_ptr_array_new ();
+  /* URIs may be invalid when entered by the user */
   for (n = 0; n < n_files; n++)
-    {
-      uri = g_file_get_uri (files[n]);
-      filename = g_file_get_path (files[n]);
-
-      /* there could be invalid uris when the call comes from gdbus */
-      if (filename)
-        {
-          g_ptr_array_add (uris, uri);
-          valid++;
-        }
-      else
-        {
-          g_message ("Invalid URI: %s", uri);
-          g_free (uri);
-        }
-
-      g_free (filename);
-    }
+    if (mousepad_util_get_path (files[n]) == NULL)
+      {
+        /* inform the user */
+        uri = g_file_get_uri (files[n]);
+        g_message ("Invalid URI: %s", uri);
+        g_free (uri);
+      }
+    else
+      valid_files[valid++] = files[n];
 
   if (valid > 0)
     {
-      /* get the underlying filename array */
-      g_ptr_array_add (uris, NULL);
-      data = g_ptr_array_free (uris, FALSE);
-
       /* open the files in tabs */
       if (application->opening_mode != WINDOW)
         {
@@ -809,12 +834,13 @@ mousepad_application_open (GApplication  *gapplication,
           window = mousepad_application_get_window_for_files (application);
 
           /* open the files */
-          opened = mousepad_window_open_files (MOUSEPAD_WINDOW (window), (gchar **) data);
+          opened = mousepad_window_open_files (MOUSEPAD_WINDOW (window), valid_files, valid,
+                                               application->encoding, FALSE);
 
           /* if at least one file was finally opened, show the window */
           if (opened > 0)
             gtk_widget_show (window);
-          else
+          else if (GTK_IS_WIDGET (window))
             gtk_widget_destroy (window);
         }
       /* open the files in windows */
@@ -826,22 +852,17 @@ mousepad_application_open (GApplication  *gapplication,
               window = mousepad_application_create_window (application);
 
               /* open the file */
-              temp_uri[0] = data[n];
-              opened = mousepad_window_open_files (MOUSEPAD_WINDOW (window), temp_uri);
+              opened = mousepad_window_open_files (MOUSEPAD_WINDOW (window), valid_files + n, 1,
+                                                   application->encoding, FALSE);
 
               /* if the file was finally opened, show the window */
               if (opened > 0)
                 gtk_widget_show (window);
-              else
+              else if (GTK_IS_WIDGET (window))
                 gtk_widget_destroy (window);
             }
         }
-
-      /* cleanup */
-      g_strfreev ((gchar **) data);
     }
-  else
-    g_ptr_array_free (uris, TRUE);
 }
 
 
@@ -1285,7 +1306,25 @@ mousepad_application_action_quit (GSimpleAction *action,
                                   GVariant      *value,
                                   gpointer       data)
 {
-  g_application_quit (G_APPLICATION (data));
+  GList    *windows, *window;
+  GAction  *close_action;
+  GVariant *v_succeed;
+  gboolean  succeed;
+
+  /* try to close all windows, abort at the first failure */
+  windows = g_list_copy (gtk_application_get_windows (GTK_APPLICATION (data)));
+  for (window = windows; window != NULL; window = window->next)
+    {
+      close_action = g_action_map_lookup_action (G_ACTION_MAP (window->data), "file.close-window");
+      g_action_activate (close_action, NULL);
+      v_succeed = g_action_get_state (close_action);
+      succeed = g_variant_get_int32 (v_succeed);
+      g_variant_unref (v_succeed);
+      if (! succeed)
+        break;
+    }
+
+  g_list_free (windows);
 }
 
 
