@@ -1,3 +1,20 @@
+/*
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; either version 2 of the License, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+ * Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+#include <mousepad/mousepad-private.h>
 #include <mousepad/mousepad-close-button.h>
 
 
@@ -52,10 +69,7 @@ mousepad_close_button_init (MousepadCloseButton *button)
   gtk_container_add (GTK_CONTAINER (button), image);
   gtk_widget_show (image);
 
-  g_object_set (G_OBJECT (button),
-                "relief", GTK_RELIEF_NONE,
-                "focus-on-click", FALSE,
-                NULL);
+  g_object_set (button, "relief", GTK_RELIEF_NONE, "focus-on-click", FALSE, NULL);
 }
 
 
