@@ -112,26 +112,39 @@ mousepad_replace_dialog_class_init (MousepadReplaceDialogClass *klass)
 
   /* add a reverse-activate signal to GtkEntry */
   entry_class = g_type_class_ref (GTK_TYPE_ENTRY);
+  binding_set = gtk_binding_set_by_class (entry_class);
   if (G_LIKELY (g_signal_lookup ("reverse-activate", GTK_TYPE_ENTRY) == 0))
     {
-      /* install the signal */
-      g_signal_new ("reverse-activate",
-                   GTK_TYPE_ENTRY,
-                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-                   0, NULL, NULL,
-                   g_cclosure_marshal_VOID__VOID,
-                   G_TYPE_NONE, 0);
-      binding_set = gtk_binding_set_by_class (entry_class);
+      g_signal_new ("reverse-activate", GTK_TYPE_ENTRY, G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                    0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
       gtk_binding_entry_add_signal (binding_set, GDK_KEY_Return,
                                     GDK_SHIFT_MASK, "reverse-activate", 0);
       gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Enter,
                                     GDK_SHIFT_MASK, "reverse-activate", 0);
     }
+
+  /* add a select-all signal to GtkEntry */
+  if (G_LIKELY (g_signal_lookup ("select-all", GTK_TYPE_ENTRY) == 0))
+    {
+      g_signal_new ("select-all", GTK_TYPE_ENTRY, G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                    0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+      gtk_binding_entry_add_signal (binding_set, GDK_KEY_a, GDK_CONTROL_MASK, "select-all", 0);
+    }
+
+  /* cleanup */
   g_type_class_unref (entry_class);
 }
 
 
 
+static void
+mousepad_replace_dialog_entry_select_all (GtkEntry *entry)
+{
+  gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
+}
+
+
+
 static void
 mousepad_replace_dialog_bind_setting (MousepadReplaceDialog *dialog,
                                       const gchar           *path,
@@ -165,15 +178,54 @@ mousepad_replace_dialog_update_label (MousepadReplaceDialog *dialog,
 static void
 mousepad_replace_dialog_post_init (MousepadReplaceDialog *dialog)
 {
+  GtkApplication   *application;
+  GtkWindow        *window;
+  GtkBindingSet    *binding_set;
+  GdkModifierType   accel_mods;
+  guint             n, accel_key;
+  gchar           **accels;
+  const gchar      *actions[] = { "win.edit.cut", "win.edit.copy", "win.edit.paste",
+                                  "win.edit.select-all" };
+  const gchar      *signals[] = { "cut-clipboard", "copy-clipboard", "paste-clipboard",
+                                  "select-all" };
+
   /* disconnect this handler */
   mousepad_disconnect_by_func (dialog, mousepad_replace_dialog_post_init, NULL);
 
+  /* get the transient parent window and the application */
+  window = gtk_window_get_transient_for (GTK_WINDOW (dialog));
+  application = gtk_window_get_application (window);
+
   /* connect to the "search-completed" parent window signal */
-  g_signal_connect_object (gtk_window_get_transient_for (GTK_WINDOW (dialog)),
-                           "search-completed",
+  g_signal_connect_object (window, "search-completed",
                            G_CALLBACK (mousepad_replace_dialog_search_completed),
                            dialog, G_CONNECT_SWAPPED);
 
+  /* make text entries keybindings consistent with those of the text view */
+  binding_set = gtk_binding_set_by_class (g_type_class_peek (GTK_TYPE_ENTRY));
+
+  accels = gtk_application_get_accels_for_action (application, "win.edit.delete-selection");
+  if (accels[0] != NULL)
+    {
+      gtk_accelerator_parse (accels[0], &accel_key, &accel_mods);
+      gtk_binding_entry_remove (binding_set, accel_key, accel_mods);
+      gtk_binding_entry_add_signal (binding_set, accel_key, accel_mods, "delete-from-cursor",
+                                    2, GTK_TYPE_DELETE_TYPE, GTK_DELETE_CHARS, G_TYPE_INT, 1);
+      g_strfreev (accels);
+    }
+
+  for (n = 0; n < G_N_ELEMENTS (actions); n++)
+    {
+      accels = gtk_application_get_accels_for_action (application, actions[n]);
+      if (accels[0] != NULL)
+        {
+          gtk_accelerator_parse (accels[0], &accel_key, &accel_mods);
+          gtk_binding_entry_remove (binding_set, accel_key, accel_mods);
+          gtk_binding_entry_add_signal (binding_set, accel_key, accel_mods, signals[n], 0);
+          g_strfreev (accels);
+        }
+    }
+
   /* give the dialog its definite size by setting a fake occurrences label */
   gtk_entry_grab_focus_without_selecting (GTK_ENTRY (dialog->search_entry));
   gtk_entry_set_text (GTK_ENTRY (dialog->search_entry), "fake-text");
@@ -254,6 +306,8 @@ mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
                             G_CALLBACK (mousepad_replace_dialog_entry_activate), dialog);
   g_signal_connect_swapped (dialog->search_entry, "reverse-activate",
                             G_CALLBACK (mousepad_replace_dialog_entry_reverse_activate), dialog);
+  g_signal_connect (dialog->search_entry, "select-all",
+                    G_CALLBACK (mousepad_replace_dialog_entry_select_all), NULL);
 
   /* bind the sensitivity of the find and replace buttons to the search text length */
   g_object_bind_property (dialog->search_entry, "text-length",
@@ -280,6 +334,8 @@ mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
 
   /* store as an entry widget */
   dialog->replace_entry = gtk_bin_get_child (GTK_BIN (combo));
+  g_signal_connect (dialog->replace_entry, "select-all",
+                    G_CALLBACK (mousepad_replace_dialog_entry_select_all), NULL);
 
   /* search direction */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
