@@ -35,15 +35,19 @@ GType           mousepad_search_bar_get_type        (void) G_GNUC_CONST;
 
 GtkWidget      *mousepad_search_bar_new             (void);
 
-GtkEditable    *mousepad_search_bar_entry           (MousepadSearchBar *bar);
-
 void            mousepad_search_bar_focus           (MousepadSearchBar *bar);
 
 void            mousepad_search_bar_find_next       (MousepadSearchBar *bar);
 
 void            mousepad_search_bar_find_previous   (MousepadSearchBar *bar);
 
-void            mousepad_search_bar_set_text        (MousepadSearchBar *bar, gchar *text);
+void            mousepad_search_bar_page_switched   (MousepadSearchBar *bar,
+                                                     GtkTextBuffer     *old_buffer,
+                                                     GtkTextBuffer     *new_buffer,
+                                                     gboolean           search);
+
+void            mousepad_search_bar_set_text        (MousepadSearchBar *bar,
+                                                     const gchar       *text);
 
 G_END_DECLS
 
