@@ -22,6 +22,11 @@
 G_BEGIN_DECLS
 
 /* Setting names */
+#define MOUSEPAD_SETTING_DEFAULT_ENCODING             "preferences.file.default-encoding"
+#define MOUSEPAD_SETTING_ADD_LAST_EOL                 "preferences.file.add-last-end-of-line"
+#define MOUSEPAD_SETTING_MAKE_BACKUP                  "preferences.file.make-backup"
+#define MOUSEPAD_SETTING_MONITOR_CHANGES              "preferences.file.monitor-changes"
+#define MOUSEPAD_SETTING_MONITOR_DISABLING_TIMER      "preferences.file.monitor-disabling-timer"
 #define MOUSEPAD_SETTING_AUTO_INDENT                  "preferences.view.auto-indent"
 #define MOUSEPAD_SETTING_FONT                         "preferences.view.font-name"
 #define MOUSEPAD_SETTING_USE_DEFAULT_FONT             "preferences.view.use-default-monospace-font"
