@@ -1089,3 +1089,33 @@ mousepad_util_pango_font_description_to_css (const PangoFontDescription *font_de
 #undef ADD_KEYVAL
 #undef ADD_KEYVAL_PRINTF
 }
+
+
+
+/* to be replaced by g_file_peek_path() when bumping GLib minimal version to 2.56 or higher */
+const gchar *
+mousepad_util_get_path (GFile *file)
+{
+#if ! GLIB_CHECK_VERSION (2, 56, 0)
+  gchar       *path;
+  const gchar *intern_path;
+#endif
+
+  g_return_val_if_fail (G_IS_FILE (file), NULL);
+
+#if GLIB_CHECK_VERSION (2, 56, 0)
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
+  return g_file_peek_path (file);
+
+G_GNUC_END_IGNORE_DEPRECATIONS
+#else
+
+  path = g_file_get_path (file);
+  intern_path = g_intern_string (path);
+  g_free (path);
+
+  return intern_path;
+
+#endif
+}
